<?php

namespace Ivoz\Ast\Domain\Model\Queue;

use Ivoz\Core\Domain\Model\LoggableEntityInterface;

interface QueueInterface extends LoggableEntityInterface
{
    /**
     * @codeCoverageIgnore
     * @return array
     */
    public function getChangeSet();

    /**
     * Get name
     *
     * @return string
     */
    public function getName(): string;

    /**
     * Get periodicAnnounce
     *
     * @return string | null
     */
    public function getPeriodicAnnounce();

    /**
     * Get periodicAnnounceFrequency
     *
     * @return integer | null
     */
    public function getPeriodicAnnounceFrequency();

    /**
     * Get announcePosition
     *
     * @return string | null
     */
    public function getAnnouncePosition();

    /**
     * Get announceFrequency
     *
     * @return integer | null
     */
    public function getAnnounceFrequency();

    /**
     * Get timeout
     *
     * @return integer | null
     */
    public function getTimeout();

    /**
     * Get autopause
     *
     * @return string
     */
    public function getAutopause(): string;

    /**
     * Get ringinuse
     *
     * @return string
     */
    public function getRinginuse(): string;

    /**
     * Get wrapuptime
     *
     * @return integer | null
     */
    public function getWrapuptime();

    /**
     * Get maxlen
     *
     * @return integer | null
     */
    public function getMaxlen();

    /**
     * Get strategy
     *
     * @return string | null
     */
    public function getStrategy();

    /**
     * Get weight
     *
     * @return integer | null
     */
    public function getWeight();

    /**
     * Get queue
     *
     * @return \Ivoz\Provider\Domain\Model\Queue\QueueInterface
     */
    public function getQueue();

    /**
     * @return bool
     */
    public function isInitialized(): bool;
}
